@@ -15,7 +15,9 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 
-	v1 "github.com/openshift/cluster-samples-operator/pkg/apis/samples/v1"
+	v1 "github.com/openshift/api/samples/v1"
+	"github.com/openshift/cluster-samples-operator/pkg/metrics"
+	"github.com/openshift/cluster-samples-operator/pkg/util"
 )
 
 const (
@@ -61,14 +63,6 @@ type ClusterOperatorWrapper interface {
 	Create(state *configv1.ClusterOperator) (err error)
 }
 
-// IsNonX86Arch let's us know if this is something other than x86_64/amd like s390x or ppc
-func IsNonX86Arch(cfg *v1.Config) bool {
-	if len(cfg.Spec.Architectures) > 0 && cfg.Spec.Architectures[0] != v1.AMDArchitecture && cfg.Spec.Architectures[0] != v1.X86Architecture {
-		return true
-	}
-	return false
-}
-
 // this method ensures that Available==true and Degraded==false, regardless of other conditions; it currently
 // allows Progressing to be explicitly set to true or false based on the scenario the caller is addressing
 func (o *ClusterOperatorHandler) setOperatorStatusWithoutInterrogatingConfig(progressing configv1.ConditionStatus, cfg *v1.Config, reason string) {
@@ -93,7 +87,7 @@ func (o *ClusterOperatorHandler) UpdateOperatorStatus(cfg *v1.Config, deletionIn
 		// will ignore errors in delete path, but we at least log them above
 		return nil
 	}
-	if IsNonX86Arch(cfg) {
+	if util.IsNonX86Arch(cfg) {
 		o.setOperatorStatusWithoutInterrogatingConfig(configv1.ConditionFalse, cfg, nonX86)
 		// will ignore errors in non-x86 path, but we at least log them above
 		return nil
@@ -101,7 +95,7 @@ func (o *ClusterOperatorHandler) UpdateOperatorStatus(cfg *v1.Config, deletionIn
 	}
 
 	errs := []error{}
-	degraded, degradedReason, degradedDetail := cfg.ClusterOperatorStatusDegradedCondition()
+	degraded, degradedReason, degradedDetail := util.ClusterOperatorStatusDegradedCondition(cfg)
 	err := o.setOperatorStatus(configv1.OperatorDegraded,
 		degraded,
 		degradedDetail,
@@ -112,8 +106,13 @@ func (o *ClusterOperatorHandler) UpdateOperatorStatus(cfg *v1.Config, deletionIn
 		errs = append(errs, err)
 		logrus.Warningf("error occurred while attempting to set degraded condition: %s", err.Error())
 	}
+	flagForMetric := false
+	if degraded == configv1.ConditionTrue {
+		flagForMetric = true
+	}
+	metrics.Degraded(flagForMetric)
 
-	available, reasonForAvailable, msgForAvailable := cfg.ClusterOperatorStatusAvailableCondition()
+	available, reasonForAvailable, msgForAvailable := util.ClusterOperatorStatusAvailableCondition(cfg)
 	// if we're setting the operator status to available, also set the operator version
 	// to the current version.
 	err = o.setOperatorStatus(configv1.OperatorAvailable,
@@ -127,7 +126,7 @@ func (o *ClusterOperatorHandler) UpdateOperatorStatus(cfg *v1.Config, deletionIn
 		logrus.Warningf("error occurred while attempting to set available condition: %s", err.Error())
 	}
 
-	progressing, reasonForProgressing, msgForProgressing := cfg.ClusterOperatorStatusProgressingCondition(degradedReason, available)
+	progressing, reasonForProgressing, msgForProgressing := util.ClusterOperatorStatusProgressingCondition(cfg, degradedReason, available)
 	err = o.setOperatorStatus(configv1.OperatorProgressing,
 		progressing,
 		msgForProgressing,
@@ -218,7 +217,7 @@ func (o *ClusterOperatorHandler) setOperatorStatus(condtype configv1.ClusterStat
 
 		state.Status.RelatedObjects = []configv1.ObjectReference{
 			{Group: v1.GroupName, Resource: "configs", Name: "cluster"},
-			{Resource: "namespaces", Name: "openshift-cluster-samples-operator"},
+			{Resource: "namespaces", Name: v1.OperatorNamespace},
 			{Resource: "namespaces", Name: "openshift"},
 		}
 
