@@ -25,11 +25,12 @@ import (
 	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 	imageset "github.com/openshift/client-go/image/clientset/versioned"
 	templateset "github.com/openshift/client-go/template/clientset/versioned"
-	samplesapi "github.com/openshift/cluster-samples-operator/pkg/apis/samples/v1"
+	samplesapi "github.com/openshift/api/samples/v1"
 	sampopclient "github.com/openshift/cluster-samples-operator/pkg/client"
 	sampleclientv1 "github.com/openshift/cluster-samples-operator/pkg/generated/clientset/versioned"
 	operator "github.com/openshift/cluster-samples-operator/pkg/operatorstatus"
 	"github.com/openshift/cluster-samples-operator/pkg/stub"
+	"github.com/openshift/cluster-samples-operator/pkg/util"
 	kubeset "k8s.io/client-go/kubernetes"
 )
 
@@ -88,7 +89,7 @@ func setupClients(t *testing.T) {
 }
 
 func dumpPod(t *testing.T) {
-	podClient := kubeClient.CoreV1().Pods("openshift-cluster-samples-operator")
+	podClient := kubeClient.CoreV1().Pods(samplesapi.OperatorNamespace)
 	podList, err := podClient.List(metav1.ListOptions{})
 	if err != nil {
 		t.Fatalf("error list pods %v", err)
@@ -98,17 +99,20 @@ func dumpPod(t *testing.T) {
 		t.Logf("dumpPods looking at pod %s in phase %s", pod.Name, pod.Status.Phase)
 		if strings.HasPrefix(pod.Name, "cluster-samples-operator") &&
 			pod.Status.Phase == corev1.PodRunning {
-			req := podClient.GetLogs(pod.Name, &corev1.PodLogOptions{})
-			readCloser, err := req.Stream()
-			if err != nil {
-				t.Fatalf("error getting pod logs %v", err)
+			for _, container := range pod.Spec.Containers {
+				req := podClient.GetLogs(pod.Name, &corev1.PodLogOptions{Container: container.Name})
+				readCloser, err := req.Stream()
+				if err != nil {
+					t.Fatalf("error getting pod logs for container %s: %s", container.Name, err.Error())
+				}
+				b, err := ioutil.ReadAll(readCloser)
+				if err != nil {
+					t.Fatalf("error reading pod stream %s", err.Error())
+				}
+				podLog := string(b)
+				t.Logf("pod logs for container %s:  %s", container.Name, podLog)
+
 			}
-			b, err := ioutil.ReadAll(readCloser)
-			if err != nil {
-				t.Fatalf("error reading pod stream %v", err)
-			}
-			podLog := string(b)
-			t.Logf("pod logs:  %s", podLog)
 		}
 	}
 }
@@ -175,10 +179,10 @@ func verifyConditionsCompleteSamplesAdded(t *testing.T) error {
 			t.Logf("%v", err)
 			return false, nil
 		}
-		if cfg.ConditionTrue(samplesapi.SamplesExist) &&
-			cfg.ConditionTrue(samplesapi.ConfigurationValid) &&
-			cfg.ConditionTrue(samplesapi.ImportCredentialsExist) &&
-			cfg.ConditionFalse(samplesapi.ImageChangesInProgress) {
+		if util.ConditionTrue(cfg, samplesapi.SamplesExist) &&
+			util.ConditionTrue(cfg, samplesapi.ConfigurationValid) &&
+			util.ConditionTrue(cfg, samplesapi.ImportCredentialsExist) &&
+			util.ConditionFalse(cfg, samplesapi.ImageChangesInProgress) {
 			return true, nil
 		}
 
@@ -194,8 +198,8 @@ func verifyConditionsCompleteSamplesRemoved(t *testing.T) error {
 			t.Logf("%v", err)
 			return false, nil
 		}
-		if cfg.ConditionFalse(samplesapi.SamplesExist) &&
-			cfg.ConditionFalse(samplesapi.ImageChangesInProgress) {
+		if util.ConditionFalse(cfg, samplesapi.SamplesExist) &&
+			util.ConditionFalse(cfg, samplesapi.ImageChangesInProgress) {
 			return true, nil
 		}
 
@@ -406,7 +410,7 @@ func verifyImageChangesInProgress(t *testing.T) {
 			t.Logf("%v", err)
 			return false, nil
 		}
-		if cfg.ConditionTrue(samplesapi.ImageChangesInProgress) {
+		if util.ConditionTrue(cfg, samplesapi.ImageChangesInProgress) {
 			return true, nil
 		}
 		return false, nil
@@ -497,7 +501,7 @@ func verifyConfigurationValid(t *testing.T, status corev1.ConditionStatus) {
 			t.Logf("%v", e)
 			return false, nil
 		}
-		if cfg.Condition(samplesapi.ConfigurationValid).Status == status {
+		if util.Condition(cfg, samplesapi.ConfigurationValid).Status == status {
 			return true, nil
 		}
 		return false, nil
@@ -900,7 +904,7 @@ func TestSpecManagementStateField(t *testing.T) {
 			t.Logf("%v", err)
 			return false, nil
 		}
-		if cfg.Condition(samplesapi.ImageChangesInProgress).LastUpdateTime.After(now.Time) {
+		if util.Condition(cfg, samplesapi.ImageChangesInProgress).LastUpdateTime.After(now.Time) {
 			return true, nil
 		}
 		return false, nil
@@ -1039,7 +1043,7 @@ func coreTestUpgrade(t *testing.T) {
 	t.Logf("current version %s version for upgrade %s", cfg.Status.Version, newVersion)
 
 	// update env to trigger upgrade
-	depClient := kubeClient.AppsV1().Deployments("openshift-cluster-samples-operator")
+	depClient := kubeClient.AppsV1().Deployments(samplesapi.OperatorNamespace)
 	err = wait.PollImmediate(1*time.Second, 1*time.Minute, func() (bool, error) {
 		dep, err := depClient.Get("cluster-samples-operator", metav1.GetOptions{})
 		if err != nil {
@@ -1077,7 +1081,7 @@ func coreTestUpgrade(t *testing.T) {
 	if cfg.Status.ManagementState == operatorsv1api.Managed {
 		err = wait.PollImmediate(1*time.Second, 1*time.Minute, func() (bool, error) {
 			cfg := verifyOperatorUp(t)
-			if cfg.ConditionTrue(samplesapi.MigrationInProgress) {
+			if util.ConditionTrue(cfg, samplesapi.MigrationInProgress) {
 				return true, nil
 			}
 			return false, nil
